/***********************************************************

File Name :
        ew2msring.h

Programmer:
        Phil Maechling

Description:
        EW to MiniSeed
        This is an earthworm client which is sits on a Tracebuf ring
        and input tracebuf packets. It then repackages them up into
        MiniSeed packets and multicasts them out onto a multicast net.

Creation Date:
        9 March 1999


Usage Notes:



Modification History:


	25 Jun 2001 - paulf - added RestartOnGap option. If present, this
		indicates that we are using analog channels and we should go into
		a restart mode upon finding a time gap
	22 Dec 2003 - paulf - added future_data_threshold option to specify the
		number of seconds in the future data are allowed to be m'casted.
		Any data in the future beyond this time are thrown away and a log
		mesage is made.

**********************************************************/
#ifndef ew2msring_H
#define ew2msring_H

#include "earthworm.h"
#include "transport.h"
#include "seismic.h"

typedef struct {
   long InKey;                    /* Key to ring where waveforms live */
   char ChannelFile[MAX_CHARS_IN_FILE_NAME]; 
				  /* Name of file with valid channels list */
   char RingServerName[MAX_CHARS_IN_FILE_LINE];
				  /* Name and optional port for ring. */
   int  HeartbeatInt;             /* Heartbeat interval in seconds */
   int  Debug;                    /* If 1, print debug messages */
   int  CompressionLevel;         /* 1 or 2 (Steim 1 or Steim 2 only */
   int  SamplesPerSend;           /* Target number of samples to buffer before */
				  /* compressing then and sending them. */
				  /* Range 1 - N (100-400 recommended) */
   int  NSlots;			  /* Number of slots for multicasting. */

   double future_data_threshold;  /* throw away any tracebufs that are X seconds 
				     in the future */
   int RestartOnGap;	          /* go into restart mode upon finding a gap */
   unsigned char MyModId;         /* Module id of this program */
   SHM_INFO InRegion;             /* Info structure for input region */
} GPARM;

typedef struct {
   unsigned char MyInstId;        /* Local installation */
   unsigned char GetThisInstId;   /* Get messages from this inst id */
   unsigned char GetThisModId;    /* Get messages from this module */
   unsigned char TypeHeartBeat;
   unsigned char TypeError;
   unsigned char TypeWaveform;    /* Waveform buffer for data input */
} EWH;

int  GetEwh( EWH * );

#endif
