#include <iostream>
#include <stdio.h>
#include <string.h>
extern "C" {
#include <kom.h>
#include <earthworm.h>
}
#include "ew2msring.h"
#include "ew2msring_config.h"

#define ncommand 9         /* Number of commands in the config file */
#define FUTURE_SECS_CHECK_DEFAULT 10.0


 /***********************************************************************
  *                              GetConfig()                            *
  *             Processes command file using kom.c functions.           *
  *               Returns -1 if any errors are encountered.             *
  ***********************************************************************/

int GetConfig( char *config_file, GPARM *Gparm )
{
   char     init[ncommand];     /* Flags, one for each command */
   int      nmiss;              /* Number of commands that were missed */
   int      nfiles;
   int      i;

	Gparm->RestartOnGap = 0;
	Gparm->future_data_threshold = FUTURE_SECS_CHECK_DEFAULT;	
/* Set to zero one init flag for each required command
   ***************************************************/
   for ( i = 0; i < ncommand; i++ )
      init[i] = 0;

/* Open the main configuration file
   ********************************/
   nfiles = k_open( config_file );
   if ( nfiles == 0 )
   {
      fprintf( stderr, "ew2msring: Error opening configuration file <%s>\n",
               config_file );
      return -1;
   }

/* Process all nested configuration files
   **************************************/
   while ( nfiles > 0 )          /* While there are config files open */
   {
      while ( k_rd() )           /* Read next line from active file  */
      {
         int  success;
         char *com;
         char *str;

         com = k_str();          /* Get the first token from line */

         if ( !com ) continue;             /* Ignore blank lines */
         if ( com[0] == '#' ) continue;    /* Ignore comments */

/* Open another configuration file
   *******************************/
         if ( com[0] == '@' )
         {
            success = nfiles + 1;
            nfiles  = k_open( &com[1] );
            if ( nfiles != success )
            {
               fprintf( stderr, "ew2msring: Error opening command file <%s>.\n",
                        &com[1] );
               return -1;
            }
            continue;
         }

/* Read configuration parameters
   *****************************/

         else if ( k_its( "MyModuleId" ) )
         {
            if ( str = k_str() )
            {
               if ( GetModId(str, &Gparm->MyModId) == -1 )
               {
                  fprintf( stderr, "ew2msring: Invalid MyModId <%s>.\n", str );
                  return -1;
               }
            }
            init[0] = 1;
         }

         else if ( k_its( "RingName" ) )
         {
            if ( str = k_str() )
            {
               if( (Gparm->InKey = GetKey(str)) == -1 )
               {
                  fprintf( stderr, "ew2msring: Invalid InRing name <%s>. Exiting.\n", str );
                  return -1;
               }
            }
            init[1] = 1;
         }

         else if ( k_its( "SamplesPerSend" ) )
         {
            Gparm->SamplesPerSend = k_int();

	    if ( (Gparm->SamplesPerSend < 100) ||
	         (Gparm->SamplesPerSend > 1000) )
            {
              std::cout << "Invalid number of Samples per send " <<
	         Gparm->SamplesPerSend << " : Setting it to 100." << std::endl;
	      Gparm->SamplesPerSend = 100;

            }
            init[2] = 1;
         }

         else if ( k_its( "Debug" ) )
         {
            Gparm->Debug = k_int();
            init[3] = 1;
         }

         else if ( k_its( "HeartBeatInt" ) )
         {
            Gparm->HeartbeatInt = k_int();
            init[4] = 1;
         }

         else if ( k_its( "ChannelFile" ) )
         {
            if ( str = k_str() )
            {
               strcpy( Gparm->ChannelFile, str );
               init[5] = 1;
            }
         }

         else if ( k_its( "RingServerName" ) )
         {
            if ( str = k_str() )
            {
               strcpy( Gparm->RingServerName, str );
               init[6] = 1;
            }
         }

         else if ( k_its( "RestartOnGap" ) )
	 {
		Gparm->RestartOnGap = 1;
	 }
         else if ( k_its( "FutureThreshold" ) )
	 {
	    Gparm->future_data_threshold = (double) k_val();
	    if ( (Gparm->future_data_threshold <= 0.0) )
            {
              std::cout << "Invalid number of Seconds for Future Data Threshold" <<
	         Gparm->future_data_threshold << " : Setting it to "<<FUTURE_SECS_CHECK_DEFAULT
		 << std::endl;
	      Gparm->future_data_threshold = FUTURE_SECS_CHECK_DEFAULT;

            }
	 }
         else if ( k_its( "CompressionLevel") )
         {
            Gparm->CompressionLevel = k_int();
	    if ( (Gparm->CompressionLevel < 1) ||
	         (Gparm->CompressionLevel > 2) )
            {
              std::cout << "Invalid CompressionLevel " << 
	         Gparm->CompressionLevel << " : Setting CompressionLevel to 1." << std::endl;
	      Gparm->CompressionLevel = 1;

            }
            init[7] = 1;
         }

         else if ( k_its( "NSlots") )
         {
            Gparm->NSlots = k_int();
	    if ( (Gparm->NSlots < 1) ||
	         (Gparm->NSlots > 100) )
            {
              std::cout << "Invalid number of starting slots " << 
	         Gparm->NSlots << " : Setting it to NSlots 10." << std::endl;
	      Gparm->NSlots = 10;

            }
            init[8] = 1;
         }




/* An unknown parameter was encountered
   ************************************/
         else
         {
            fprintf( stderr, "ew2msring: <%s> unknown parameter in <%s>\n",
                    com, config_file );
            continue;
         }

/* See if there were any errors processing the command
   ***************************************************/
         if ( k_err() )
         {
            fprintf( stderr, "ew2msring: Bad <%s> command in <%s>.\n", com,
                     config_file );
            return -1;
         }
      }
      nfiles = k_close();
   }

/* After all files are closed, check flags for missed commands
   ***********************************************************/
   nmiss = 0;
   for ( i = 0; i < ncommand; i++ )
      if ( !init[i] )
         nmiss++;

   if ( nmiss > 0 )
   {
      fprintf( stderr, "ew2msring: ERROR, no " );
      if ( !init[0] ) fprintf( stderr, "<MyModuleId> " );
      if ( !init[1] ) fprintf( stderr, "<RingName> " );
      if ( !init[2] ) fprintf( stderr, "<SamplesPerSend> " );
      if ( !init[3] ) fprintf( stderr, "<Debug> " );
      if ( !init[4] ) fprintf( stderr, "<HeartbeatInt> " );
      if ( !init[5] ) fprintf( stderr, "<ChannelFile> " );
      if ( !init[6] ) fprintf( stderr, "<RingServerName> " );
      if ( !init[7] ) fprintf( stderr, "<CompressionLevel> " );
      if ( !init[8] ) fprintf( stderr, "<NSlots> " );
      fprintf( stderr, "command(s) in <%s>. Exiting.\n", config_file );
      return -1;
   }
   return 0;
}


 /***********************************************************************
  *                              LogConfig()                            *
  *                                                                     *
  *                   Log the configuration parameters                  *
  ***********************************************************************/

void LogConfig( GPARM *Gparm )
{
   logit( "", "\n" );
   logit( "", "MyModId:               %6u\n\n", Gparm->MyModId );
   logit( "", "RingName:              %6ld\n",   Gparm->InKey );
   logit( "", "SamplesPerSend:        %6d\n",   Gparm->SamplesPerSend);
   logit( "", "Debug:                 %6d\n",   Gparm->Debug );
   logit( "", "HeartbeatInt:          %6d\n",   Gparm->HeartbeatInt );
   logit( "", "ChannelFile:           %s\n",    Gparm->ChannelFile );
   logit( "", "RingServerName:        %s\n",    Gparm->RingServerName );
   logit( "", "CompressionLevel:      %6d\n",   Gparm->CompressionLevel);
   logit( "", "NSlots:                %6d\n",   Gparm->NSlots);
   return;
}
